@@ -4,7 +4,7 @@ from cpm.api.result import Result, OK, FAIL
 from cpm.domain.cmake.cmakelists_builder import CMakeListsBuilder
 from cpm.domain.compilation_service import CompilationService
 from cpm.domain.project_commands import DockerImageNotFound
-from cpm.domain.project.project_descriptor_parser import NotACpmProject
+from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound
 from cpm.domain.project.project_loader import ProjectLoader, InvalidTarget
 from cpm.domain.project_commands import ProjectCommands, BuildError
 
@@ -12,7 +12,7 @@ from cpm.domain.project_commands import ProjectCommands, BuildError
 def build_project(compilation_service, target='default'):
     try:
         compilation_service.build(target)
-    except NotACpmProject:
+    except ProjectDescriptorNotFound:
         return Result(FAIL, f'error: not a cpm project')
     except BuildError:
         return Result(FAIL, f'error: compilation failed')
