@@ -3,10 +3,9 @@ import argparse
 from cpm.api.result import Result
 from cpm.api.result import OK
 from cpm.api.result import FAIL
-from cpm.domain.bit_packager import BitPackager
 from cpm.domain.publish_service import PublishService
 from cpm.domain.bit_packager import PackagingFailure
-from cpm.domain.project.project_descriptor_parser import NotACpmProject
+from cpm.domain.project.project_descriptor_parser import ProjectDescriptorNotFound
 from cpm.infrastructure.cpm_hub_connector_v1 import CpmHubConnectorV1
 from cpm.infrastructure.cpm_hub_connector_v1 import InvalidCpmHubUrl
 from cpm.infrastructure.cpm_hub_connector_v1 import AuthenticationFailure
@@ -14,10 +13,13 @@ from cpm.infrastructure.cpm_hub_connector_v1 import PublicationFailure
 from cpm.infrastructure.http_client import HttpConnectionError
 
 
-def publish_project(publish_service):
+def publish_project(publish_service, publish_as_template=False):
     try:
-        publish_service.publish()
-    except NotACpmProject:
+        if publish_as_template:
+            publish_service.publish_template()
+        else:
+            publish_service.publish()
+    except ProjectDescriptorNotFound:
         return Result(FAIL, f'error: not a CPM project')
     except PackagingFailure as error:
         return Result(FAIL, f'error: {error.cause}')
@@ -38,13 +40,13 @@ def publish_project(publish_service):
 def execute(argv):
     publish_parser = argparse.ArgumentParser(prog='cpm publish', description='cpm Package Manager', add_help=False)
     publish_parser.add_argument('-s', '--repository-url', required=True, action='store', default='https://repo.cpmbits.com:8000')
+    publish_parser.add_argument('-t', '--template', required=False, action='store_true', default=False)
     publish_parser.add_argument('-d', '--dry-run', required=False, action='store_true', default=False)
     args = publish_parser.parse_args(argv)
 
-    packager = BitPackager()
     cpm_hub_connector = CpmHubConnectorV1(repository_url=args.repository_url, dry_run=args.dry_run)
-    service = PublishService(packager, cpm_hub_connector)
+    service = PublishService(cpm_hub_connector)
 
-    result = publish_project(service)
+    result = publish_project(service, publish_as_template=args.template)
 
     return result
