@@ -3,3 +3,5 @@ BUILD_DIRECTORY = 'build'
 CMAKELISTS = 'CMakeLists.txt'
 CMAKE_COMMAND = 'cmake'
 NINJA_COMMAND = 'ninja'
+DEFAULT_PROJECT_VERSION = '0.0.1'
+DEFAULT_TARGET = 'default'
