@@ -1,3 +1,4 @@
+from cpm.domain.constants import DEFAULT_TARGET
 from cpm.infrastructure import filesystem
 from cpm.domain.project.project_descriptor import TargetDescription
 from cpm.domain.project.project import Project, Target, Package, TestSuite
@@ -24,12 +25,16 @@ def compose_target(target_name, project_descriptor):
     target.include_directories.update(project_descriptor.build.includes)
     target.main = target_description.main
     target.image = target_description.image
+    target.test_image = target_description.test_image
     target.dockerfile = target_description.dockerfile
+    target.toolchain_prefix = target_description.toolchain_prefix
     target.post_build = target_description.post_build
     compose_packages(project_descriptor.build.packages, target)
     compose_packages(target_description.build.packages, target)
     for bit_description in project_descriptor.build.bits.values():
-        compose_bit(bit_description, target, target_name)
+        compose_bit(bit_description, target)
+    for bit_description in target_description.build.bits.values():
+        compose_bit(bit_description, target)
     return target
 
 
@@ -43,7 +48,7 @@ def compose_tests(target_name, project_descriptor, project):
     compose_packages(target_description.test.packages, project.test)
 
     for bit_description in project_descriptor.test.bits.values():
-        compose_bit(bit_description, project.test, target_name)
+        compose_bit(bit_description, project.test)
 
     for test_file in filesystem.find('tests', 'test_*.cpp'):
         name = test_file.split('/')[-1].split('.')[0]
@@ -61,12 +66,22 @@ def compose_packages(packages, target):
         package.include_directories = target.include_directories
 
 
-def compose_bit(bit_description, target, target_name):
+def compose_bit(bit_description, target):
     adjust_bit_packages_base_path(bit_description, bit_description.build.packages)
-    if target_name in bit_description.targets:
-        adjust_bit_packages_base_path(bit_description, bit_description.targets[target_name].build.packages)
     add_packages_to_target_includes(bit_description.build.packages, target)
-    target.bits.append(compose_target(target_name, bit_description))
+    bit_target_name = get_bit_target_name(bit_description)
+    adjust_bit_packages_base_path(bit_description, bit_description.targets[bit_target_name].build.packages)
+    add_packages_to_target_includes(bit_description.targets[bit_target_name].build.packages, target)
+    bit_target = compose_target(bit_target_name, bit_description)
+    target.bits.append(bit_target)
+    add_cflags_to_bit_packages(bit_target, bit_description.declared_bit.cflags)
+
+
+def get_bit_target_name(bit_description):
+    if bit_description.declared_bit.target in bit_description.targets:
+        return bit_description.declared_bit.target
+    else:
+        return DEFAULT_TARGET
 
 
 def adjust_bit_packages_base_path(bit_description, packages):
@@ -81,3 +96,8 @@ def add_packages_to_target_includes(packages, target):
 
 def package_include_directory(package_description):
     return filesystem.parent_directory(package_description.path)
+
+
+def add_cflags_to_bit_packages(bit_target, cflags):
+    for package in bit_target.packages:
+        package.cflags.extend(cflags)
