@@ -3,13 +3,13 @@ import mock
 
 from cpm.domain.bit_packager import BitPackager
 from cpm.domain.bit_packager import PackagingFailure
-from cpm.domain.project.project import Project, Package
+from cpm.domain.project.project_descriptor import ProjectDescriptor, TargetDescription, PackageDescription
 
 
 class TestBitPackager(unittest.TestCase):
     def test_packaging_bit_raises_exception_when_no_packages_are_declared(self):
         packager = BitPackager()
-        project = Project('cest')
+        project = ProjectDescriptor('cest')
 
         self.assertRaises(PackagingFailure, packager.pack, project, 'dist')
 
@@ -17,8 +17,8 @@ class TestBitPackager(unittest.TestCase):
     def test_packaging_bit_raises_exception_when_output_directory_exists(self, filesystem):
         filesystem.directory_exists.return_value = True
         packager = BitPackager()
-        project = Project('cest')
-        project.target.packages.append(Package('fakeit'))
+        project = ProjectDescriptor('cest')
+        project.build.packages.append(PackageDescription('fakeit'))
 
         self.assertRaises(PackagingFailure, packager.pack, project, 'dist')
 
@@ -26,9 +26,10 @@ class TestBitPackager(unittest.TestCase):
     def test_packaging_creates_output_directory(self, filesystem):
         filesystem.directory_exists.return_value = False
         packager = BitPackager()
-        project = Project('cest')
-        project.target.packages.append(Package('api'))
-        project.target.packages.append(Package('domain'))
+        project = ProjectDescriptor('cest')
+        project.targets['default'] = TargetDescription('default')
+        project.build.packages.append(PackageDescription('api'))
+        project.targets['default'].build.packages.append(PackageDescription('domain'))
 
         packager.pack(project, 'dist')
 
