@@ -84,13 +84,21 @@ class TestProjectDescriptorParser(unittest.TestCase):
             'targets': {
                 'default': {
                     'image': 'cpmbits/bender',
-                    'main': 'main.c'
+                    'main': 'main.c',
+                    'test_image': 'cpmbits/bender_test',
+                    'toolchain_prefix': 'arm-linux-gnueabi-'
                 }
             }
         }
         project = project_descriptor_parser.parse_yaml(yaml_contents)
         assert project.targets == {
-            'default': project_descriptor.TargetDescription('default', image='cpmbits/bender', main='main.c')
+            'default': project_descriptor.TargetDescription(
+                'default',
+                image='cpmbits/bender',
+                main='main.c',
+                test_image='cpmbits/bender_test',
+                toolchain_prefix='arm-linux-gnueabi-'
+            )
         }
 
     def test_parse_project_descriptor_with_target_build_compilation_plan(self):
@@ -104,7 +112,8 @@ class TestProjectDescriptorParser(unittest.TestCase):
                     'build': {
                         'packages': {
                             'arduino': {},
-                        }
+                        },
+                        'ldflags': ['-Wl,--wrap=malloc']
                     },
                     'post_build': [
                         './scripts/package.sh'
@@ -116,6 +125,7 @@ class TestProjectDescriptorParser(unittest.TestCase):
         assert project.targets['arduino'].build.packages == [
             project_descriptor.PackageDescription('arduino')
         ]
+        assert project.targets['arduino'].build.ldflags == ['-Wl,--wrap=malloc']
         assert project.build_packages() == [
             project_descriptor.PackageDescription('arduino')
         ]
@@ -142,3 +152,29 @@ class TestProjectDescriptorParser(unittest.TestCase):
         assert project.targets['arduino'].test.packages == [
             project_descriptor.PackageDescription('arduino')
         ]
+
+    def test_parse_project_descriptor_with_customized_bit_compilation(self):
+        yaml_contents = {
+            'name': 'bender bender rodriguez',
+            'version': '1.0',
+            'description': 'kill all humans',
+            'build': {
+                'packages': {
+                    'cpmhub/http': None
+                },
+                'bits': {
+                    'sqlite3': {
+                        'version': '3.32.3',
+                        'cflags': ['-DCUSTOM_BIT_DEFINE'],
+                        'target': 'arduinoNano33'
+                    }
+                }
+            }
+        }
+        project = project_descriptor_parser.parse_yaml(yaml_contents)
+        assert project.build.packages == [
+            project_descriptor.PackageDescription('cpmhub/http')
+        ]
+        assert project.build.declared_bits[0] == project_descriptor.DeclaredBit(
+            name='sqlite3', version='3.32.3', cflags=['-DCUSTOM_BIT_DEFINE'], target='arduinoNano33'
+        )
